"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var is_1 = require("../utils/is");
/**
 * Represents the DataParser class, which is responsible for parsing
 * the ASTParserTree into the data that makes sense.
 *
 * @internal
 */
var DataParser = /** @class */ (function () {
    function DataParser() {
    }
    /**
     * Parses the dataTree and returns the sensible POJO.
     * @param dataTree The parsed dataTree
     * @param defs The defs references, (Not in use currently!)
     *
     * TODO: Implementations on defs-references not yet finalized!
     */
    DataParser.parse = function (dataTree, vars) {
        if (!is_1.isParserTree(dataTree))
            return undefined;
        if (dataTree.type === 'scalar') {
            var val = dataTree.values[0];
            if (is_1.isToken(val)) {
                return val.value;
            }
            if (is_1.isUndefined(val)) {
                return;
            }
        }
        var container = dataTree.type === 'object' ? {} : [];
        if (dataTree.values.length === 0) {
            return container;
        }
        return _parse(dataTree, container, vars);
    };
    return DataParser;
}());
exports.default = DataParser;
var _parse = function (root, container, vars) {
    var _a;
    for (var index = 0; index < root.values.length; index += 1) {
        var value = root.values[index];
        if (is_1.isParserTree(value)) {
            // Object
            if (value.type === 'object') {
                container[index] = _parse(value, {}, vars);
            }
            // Array
            else if (value.type === 'array') {
                container[index] = _parse(value, [], vars);
            }
        }
        else if (is_1.isKeyVal(value)) {
            var parsedValue = null;
            if (is_1.isParserTree(value.value)) {
                parsedValue = _parse(value.value, value.value.type === 'object' ? {} : [], vars);
            }
            else if (is_1.isKeyVal(value.value)) {
                console.warn('Validate this case!');
            }
            else {
                // container[value.key] = value.value === null ? undefined : value.value.value
                parsedValue = _getValue(value.value, vars);
            }
            if (is_1.isArray(container)) {
                container.push((_a = {},
                    _a[value.key] = parsedValue,
                    _a));
            }
            else {
                container[value.key] = parsedValue;
            }
        }
        else {
            // container[index] = value === null ? undefined : value.value
            container[index] = _getValue(value, vars);
        }
    }
    return container;
};
// TODO: Handle defs!
function _getValue(value, vars) {
    if (is_1.isToken(value)) {
        if (vars && is_1.isString(value.value) && value.value.startsWith('$')) {
            var key = value.value.substr(1);
            var val = vars.get(key);
            return val ? val : value.value;
        }
        return value.value;
    }
    return;
}
//# sourceMappingURL=index.js.map