import { Node } from '../parser';
/**
 * Represents the base error class in InternetObject.
 */
export declare class InternetObjectError extends Error {
    __proto__: Error;
    /**
     * An error-code associated with the current error.
     */
    errorCode?: string;
    /**
     * A line number (in the text) where error has occured.
     * Only available with some errors while parsting text data.
     */
    lineNumber?: number;
    /**
     * Column number (in the text) where error has occured.
     * Only available with some errors while parsting text data.
     */
    columnNumber?: number;
    /**
     * The index (in the text) where error has occured.
     * Only available with some errors while parsting text data.
     */
    index?: number;
    /**
     * Initialize the new instance of `InternetObjectError`.
     *
     * @param errorCode {string} An error-code associated with this error
     * @param message {string} The error message
     * @param node {Node} The node object, for tracking line and columns. Optional
     * @param ssf {Function} The start statck function, removes the irrelavant frames from the stack trace
     */
    constructor(errorCode: string, message?: string, node?: Node, ssf?: any);
}
/**
 * Represents the syntax error in InternetObject. When this error is thrwon,
 * it suggests that a syntax in the associated object is is not correct.
 */
export declare class InternetObjectSyntaxError extends InternetObjectError {
}
/**
 * Represents the validation error in InternetObject. This error is thrown when a validation
 * issue is found while validating the internet-object data against the associated schema.
 */
export declare class InternetObjectValidationError extends InternetObjectError {
}
export declare type ErrorArgs = [string, string, Node?];
//# sourceMappingURL=io-error.d.ts.map