"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.InternetObjectValidationError = exports.InternetObjectSyntaxError = exports.InternetObjectError = void 0;
/**
 * Represents the base error class in InternetObject.
 */
var InternetObjectError = /** @class */ (function (_super) {
    __extends(InternetObjectError, _super);
    /**
     * Initialize the new instance of `InternetObjectError`.
     *
     * @param errorCode {string} An error-code associated with this error
     * @param message {string} The error message
     * @param node {Node} The node object, for tracking line and columns. Optional
     * @param ssf {Function} The start statck function, removes the irrelavant frames from the stack trace
     */
    function InternetObjectError(errorCode, message, node, ssf) {
        var _newTarget = this.constructor;
        if (message === void 0) { message = ''; }
        var _this = _super.call(this) || this;
        var errorMsg = errorCode;
        _this.errorCode = errorCode;
        _this.name = 'InternetObjectError';
        if (node) {
            _this.lineNumber = node.row;
            _this.columnNumber = node.col;
            _this.index = node.index;
            errorMsg = errorCode + " at (" + node.row + ", " + node.col + ")";
        }
        else {
            errorMsg = errorCode;
        }
        _this.message = message !== '' ? errorMsg + ": " + message : errorMsg;
        // TODO: After stability, change the SSF class
        Error.captureStackTrace(_this, InternetObjectError);
        // Error.captureStackTrace(this, ssf || InternetObject)
        _this.__proto__ = _newTarget.prototype;
        return _this;
    }
    return InternetObjectError;
}(Error));
exports.InternetObjectError = InternetObjectError;
/**
 * Represents the syntax error in InternetObject. When this error is thrwon,
 * it suggests that a syntax in the associated object is is not correct.
 */
var InternetObjectSyntaxError = /** @class */ (function (_super) {
    __extends(InternetObjectSyntaxError, _super);
    /**
     * Creates a new `InternetObjectSyntaxError` error.
     * @param errorCode {string} An errorCode associated with is error
     * @param message {string} The message which needs to be displayed
     * @param node {Node} The node object, required while parsing raw internet-object data or schema
     * @param ssf {Function} The start statck function, removes the irrelavant frames from the stack trace
     *
     * @internal
     */
    function InternetObjectSyntaxError(errorCode, message, node, ssf) {
        if (message === void 0) { message = ''; }
        var _this = _super.call(this, errorCode, message, node, ssf) || this;
        _this.name = 'InternetObject(SyntaxError)';
        return _this;
    }
    return InternetObjectSyntaxError;
}(InternetObjectError));
exports.InternetObjectSyntaxError = InternetObjectSyntaxError;
/**
 * Represents the validation error in InternetObject. This error is thrown when a validation
 * issue is found while validating the internet-object data against the associated schema.
 */
var InternetObjectValidationError = /** @class */ (function (_super) {
    __extends(InternetObjectValidationError, _super);
    /**
     * Creates a new `InternetObjectValidationError` error.
     * @param errorCode {string} An errorCode associated with is error
     * @param message {string} The message which needs to be displayed
     * @param node {Node} The node object, required while parsing raw internet-object data or schema
     * @param ssf {Function} The start statck function, removes the irrelavant frames from the stack trace
     *
     * @internal
     */
    function InternetObjectValidationError(errorCode, message, node, ssf) {
        if (message === void 0) { message = ''; }
        var _this = _super.call(this, errorCode, message, node, ssf) || this;
        _this.name = 'InternetObject(ValidationError)';
        return _this;
    }
    return InternetObjectValidationError;
}(InternetObjectError));
exports.InternetObjectValidationError = InternetObjectValidationError;
//# sourceMappingURL=io-error.js.map