import KeyValueCollection from './header/index';
import Schema from './header/schema';
/**
 * Represents the InternetObject. A main facade class that
 * takes care of parsing, schema processing and other
 * core responsibilities.
 *
 * @public
 */
export declare class InternetObject<T = any> {
    /**
     * Initializes the new instance of the `InternetObject`
     * @param o {string, object} An internet object text or POJO which needs to be parsed or loaded
     * @param schema {string, Schema} An optional `Schema` object or its `string` representation
     */
    constructor(o: any, schema?: string | Schema);
    /**
     * Create or define definitions.
     * @param callSite
     * @param substitutions
     */
    static doc(callSite?: any, ...substitutions: any[]): InternetObject<any>;
    /**
     * Create or define definitions.
     * @param callSite
     * @param substitutions
     */
    static defs(callSite?: any, ...substitutions: any[]): KeyValueCollection;
    private _header;
    /**
     * Gets the header section of the object.
     */
    get header(): KeyValueCollection;
    /**
     * Gets the schema associated with the object.
     */
    get schema(): Schema;
    serialize(): string;
    private _data;
    /**
     * Gets the data section of the object
     */
    get data(): T;
}
//# sourceMappingURL=internet-object.d.ts.map