"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var io_error_codes_1 = require("../errors/io-error-codes");
var index_1 = require("../data/index");
var index_2 = require("../utils/index");
var typedef_registry_1 = require("./typedef-registry");
var utils_1 = require("./utils");
var io_error_1 = require("../errors/io-error");
var is_1 = require("../utils/is");
/**
 * Represents the AnyTypeDef which is reponsible for parsing,
 * validating, loading and serializing any values.
 */
var AnyDef = /** @class */ (function () {
    function AnyDef() {
    }
    /**
     * Returns the type this instance is going to handle.
     * Always returns "any"
     */
    AnyDef.prototype.getType = function () {
        return 'any';
    };
    /**
     * Parses any value in IO format into JavaScript.
     */
    AnyDef.prototype.parse = function (data, memberDef, vars) {
        var validatedData = utils_1.doCommonTypeCheck(memberDef, data, data);
        if (validatedData !== data || validatedData === null || validatedData === undefined) {
            return validatedData;
        }
        if (is_1.isToken(data)) {
            var value = data.value;
            if (vars) {
                var valueFound = vars.getV(value);
                value = valueFound || value;
            }
            return value;
        }
        if (is_1.isParserTree(data)) {
            return index_1.default.parse(data, vars);
        }
        // TODO: check this case
        console.assert(false, 'Check this case!');
        console.warn(data, memberDef);
    };
    /**
     * Loads the JavaScript values.
     */
    AnyDef.prototype.load = function (data, memberDef) {
        var validatedData = utils_1.doCommonTypeCheck(memberDef, data);
        if (validatedData !== data)
            return validatedData;
        return data;
    };
    /**
     * Serializes any value from JavaScript into IO format.
     */
    AnyDef.prototype.serialize = function (data, memberDef, isRoot) {
        if (isRoot === void 0) { isRoot = false; }
        return _serialize(data, memberDef, isRoot);
    };
    return AnyDef;
}());
exports.default = AnyDef;
var _serialize = function (data, memberDef, isRoot) {
    var validatedData = utils_1.doCommonTypeCheck(memberDef, data);
    if (validatedData !== data)
        return validatedData;
    // data is a string
    if (is_1.isString(data)) {
        return typedef_registry_1.TypedefRegistry.get('string').serialize(data, __assign(__assign({}, memberDef), { type: 'string' }));
    }
    // data is a number
    if (is_1.isNumber(data)) {
        return typedef_registry_1.TypedefRegistry.get('number').serialize(data, __assign(__assign({}, memberDef), { type: 'number' }));
    }
    // data is a boolean
    if (is_1.isBoolean(data)) {
        return typedef_registry_1.TypedefRegistry.get('bool').serialize(data, __assign(__assign({}, memberDef), { type: 'bool' }));
    }
    // data is an array
    if (is_1.isArray(data)) {
        return _serializeArray(data, __assign({}, memberDef), isRoot);
    }
    // data is null
    if (data === null) {
        return 'N';
    }
    // data is undefined
    if (data === undefined) {
        return '';
    }
    // data is a plain javascript object
    if (is_1.isPlainObject(data)) {
        return _serializeObject(data, memberDef, isRoot);
    }
    throw new io_error_1.InternetObjectError(io_error_codes_1.default.invalidObject);
};
var _serializeObject = function (data, memberDef, isRoot) {
    var serialized = [];
    Object.keys(data).forEach(function (key) {
        var value = data[key];
        var path = memberDef.path;
        serialized.push(_serialize(value, __assign(__assign({}, memberDef), { path: index_2.appendPath(key, path) }), false));
    });
    if (isRoot) {
        return serialized.join(',').replace(/,+$/g, '');
    }
    return "{" + serialized.join(',').replace(/,+$/g, '') + "}";
};
var _serializeArray = function (data, memberDef, isRoot) {
    var serialized = [];
    var path = memberDef.path;
    data.forEach(function (value) {
        serialized.push(_serialize(value, __assign(__assign({}, memberDef), { path: index_2.appendPath('[', path) }), false));
    });
    if (isRoot) {
        return serialized.join(',');
    }
    return "[" + serialized.join(',') + "]";
};
//# sourceMappingURL=any.js.map