"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var io_error_codes_1 = require("../errors/io-error-codes");
var is_1 = require("../utils/is");
var io_error_1 = require("../errors/io-error");
var typedef_registry_1 = require("./typedef-registry");
var utils_1 = require("./utils");
// age?: { number, true, 10, min:10, max:20}
/**
 * Represents the `array`, performs following validations.
 * - Value is an array
 * - Value is optional
 * - Value is nullable
 * - array is <= schema.maxLength
 * - array is >= schema.minLength
 * - Value is in choices
 */
var ArrayDef = /** @class */ (function () {
    function ArrayDef() {
        this._keys = null;
        this.parse = function (data, memberDef, vars) {
            if (is_1.isParserTree(data)) {
                return _process('parse', memberDef, data.values, data, vars);
            }
            else if (data === undefined) {
                return _process('parse', memberDef, undefined, data, vars);
            }
            throw new Error('invalid-value');
        };
        this.load = function (data, memberDef) {
            var value = data === undefined ? undefined : data;
            return _process('load', memberDef, value, data);
            // const validatedData = doCommonTypeCheck(memberDef)
            // if (validatedData !== data) return validatedData
            // if (!isArray(data)) throw new Error("invalid-value")
            // const schema = memberDef.schema
            // let typeDef:TypeDef | undefined
            // if (schema.type) {
            //   typeDef = TypedefRegistry.get(schema.type)
            // }
            // else {
            //   console.assert(false, "Invalid Case: Array schema must have a type attribute!")
            //   throw new Error("Verify this case!")
            // }
            // const array:any = []
            // data.forEach((item:any) => {
            //   if(typeDef !== undefined) {
            //     const value = typeDef.load(item, schema)
            //     array.push(value)
            //   }
            //   else {
            //     // TODO: Improve this error
            //     throw ErrorCodes.invalidType
            //   }
            // })
            // return array
        };
        this.serialize = function (data, memberDef) {
            if (memberDef.type !== 'array') {
                throw new io_error_1.InternetObjectError(io_error_codes_1.default.invalidArray);
            }
            var validatedData = utils_1.doCommonTypeCheck(memberDef, data);
            if (validatedData !== data) {
                // TODO: Test, when the validated data is not same,
                // can occur in situation when the default value is repaced
                data = validatedData;
            }
            else if (validatedData === undefined) {
                // When undefined allowed
                return '';
            }
            else if (validatedData === null) {
                // When the null is allowed or the default value is null
                return 'N';
            }
            var serialized = [];
            var schema = memberDef.schema;
            var typeDef = typedef_registry_1.TypedefRegistry.get(schema.type);
            data.forEach(function (value) {
                serialized.push(typeDef.serialize(value, schema));
            });
            return "[" + serialized.join(',') + "]";
        };
    }
    ArrayDef.prototype.getType = function () {
        return 'array';
    };
    return ArrayDef;
}());
function _process(processingFnName, memberDef, value, node, vars) {
    var validatedData = utils_1.doCommonTypeCheck(memberDef, value, node);
    if (validatedData !== value || value === undefined)
        return validatedData;
    if (!Array.isArray(value))
        throw new Error('invalid-value');
    var schema = memberDef.schema;
    var typeDef;
    if (schema.type) {
        typeDef = typedef_registry_1.TypedefRegistry.get(schema.type);
    }
    else {
        console.assert(false, 'Invalid Case: Array schema must have a type attribute!');
        throw new Error('Verify this case!');
    }
    var array = [];
    value.forEach(function (item) {
        if (typeDef !== undefined) {
            var value_1 = typeDef[processingFnName](item, schema, vars);
            array.push(value_1);
        }
        else {
            // TODO: Improve this error
            throw io_error_codes_1.default.invalidType;
        }
    });
    return array;
}
function _invlalidChoice(key, token, min) {
    return [
        io_error_codes_1.default.invalidMinValue,
        "The \"" + key + "\" must be greater than or equal to " + min + ", Currently it is \"" + token.value + "\".",
        token
    ];
}
function _invlalidMinLength(key, token, min) {
    return [
        io_error_codes_1.default.invalidMinValue,
        "The \"" + key + "\" must be greater than or equal to " + min + ", Currently it is \"" + token.value + "\".",
        token
    ];
}
function _invlalidMaxLength(key, token, max) {
    return [
        io_error_codes_1.default.invalidMaxValue,
        "The \"" + key + "\" must be less than or equal to " + max + ", Currently it is \"" + token.value + "\".",
        token
    ];
}
exports.default = ArrayDef;
//# sourceMappingURL=array.js.map