"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var is_1 = require("../utils/is");
var utils_1 = require("./utils");
var io_error_codes_1 = require("../errors/io-error-codes");
var io_error_1 = require("../errors/io-error");
/**
 * Represents the InternetObject String, performs following validations.
 *
 * - Value is boolean
 * - Value is optional
 * - Value is nullable
 */
var BooleanDef = /** @class */ (function () {
    function BooleanDef(type) {
        if (type === void 0) { type = 'bool'; }
        this._type = type;
    }
    BooleanDef.prototype.getType = function () {
        return this._type;
    };
    BooleanDef.prototype.parse = function (data, memberDef, vars) {
        return this.validate(data, memberDef, vars);
    };
    BooleanDef.prototype.load = function (data, memberDef) {
        return this.validate(data, memberDef);
    };
    BooleanDef.prototype.serialize = function (data, memberDef) {
        var value = this.validate(data, memberDef);
        if (value === undefined)
            return '';
        return value === true ? 'T' : 'F';
    };
    BooleanDef.prototype.validate = function (data, memberDef, vars) {
        var node = is_1.isToken(data) ? data : undefined;
        var value = node ? node.value : data;
        if (vars && is_1.isString(value)) {
            var valueFound = vars.getV(value);
            value = valueFound !== undefined ? valueFound : value;
        }
        var validatedData = utils_1.doCommonTypeCheck(memberDef, value, node);
        if (validatedData !== value || validatedData === null || validatedData === undefined) {
            return validatedData;
        }
        if (!is_1.isBoolean(value)) {
            throw new io_error_1.InternetObjectValidationError(io_error_codes_1.default.notABool, 'Expecting a boolean value', node);
        }
        return value;
    };
    return BooleanDef;
}());
exports.default = BooleanDef;
//# sourceMappingURL=boolean.js.map