"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var io_error_1 = require("../errors/io-error");
var io_error_codes_1 = require("../errors/io-error-codes");
var is_1 = require("../utils/is");
var utils_1 = require("./utils");
var datetime_1 = require("../utils/datetime");
var DATETIME_TYPES = ['datetime', 'date', 'time'];
/**
 * Represents the various number related data types
 * (such as number, int, byte, int16, int32) in Internet Object.
 *
 * @internal
 */
var DateTimeDef = /** @class */ (function () {
    function DateTimeDef(type) {
        var _this = this;
        if (type === void 0) { type = 'datetime'; }
        this.serialize = function (data, memberDef) {
            var validatedData = utils_1.doCommonTypeCheck(memberDef, data);
            if (validatedData === undefined)
                return '';
            else if (validatedData === null)
                return 'N';
            if (data instanceof Date) {
                return _getSerializer(_this._type)(data);
            }
            throw new io_error_1.InternetObjectValidationError(io_error_codes_1.default.invalidDateTime, "Expecting the value of type '" + _this._type + "'");
        };
        this._type = type;
    }
    DateTimeDef.prototype.getType = function () {
        return this._type;
    };
    DateTimeDef.prototype.parse = function (data, memberDef, vars) {
        var parse = _getParser(this._type);
        var ofMatchingType = _getTypeChecker(this._type);
        var node = is_1.isToken(data) ? data : undefined;
        var value = node ? node.value : data;
        var validatedData = utils_1.doCommonTypeCheck(memberDef, value, node);
        if (validatedData === undefined || validatedData === null) {
            return validatedData;
        }
        if (vars && is_1.isString(value) && value.startsWith('$')) {
            var replaced = vars.getV(value);
            value = replaced !== undefined ? replaced : value;
        }
        var parsed = false;
        if (ofMatchingType(value)) {
            value = parse(value);
            parsed = true;
        }
        // console.warn(parsed, ofMatchingType(value), value, memberDef)
        if (parsed === false || value === null) {
            throw new io_error_1.InternetObjectValidationError(io_error_codes_1.default.invalidDateTime, "Expecting the value of type '" + this._type + "'");
        }
        return value;
    };
    DateTimeDef.prototype.load = function (data, memberDef) {
        var value = utils_1.doCommonTypeCheck(memberDef, data);
        if (value === undefined || value === null) {
            return value;
        }
        if (value instanceof Date === false) {
            throw new io_error_1.InternetObjectValidationError(io_error_codes_1.default.invalidDateTime, "Expecting the value of type '" + this._type + "'");
        }
        return value;
    };
    return DateTimeDef;
}());
function _serializeDateTime(date) {
    return datetime_1.dateToDatetimeString(date, true, true) || '';
}
function _serializeDate(date) {
    return datetime_1.dateToDateString(date, true) || '';
}
function _serializeTime(date) {
    return datetime_1.dateToTimeString(date, true) || '';
}
function _getSerializer(type) {
    if (type === 'datetime') {
        return _serializeDateTime;
    }
    else if (type === 'date') {
        return _serializeDate;
    }
    return _serializeTime;
}
function _getParser(type) {
    if (type === 'datetime') {
        return datetime_1.parseDateTime;
    }
    else if (type === 'date') {
        return datetime_1.parseDate;
    }
    return datetime_1.parseTime;
}
function _getTypeChecker(type) {
    if (type === 'datetime') {
        return is_1.isDateTimeString;
    }
    else if (type === 'date') {
        return is_1.isDateString;
    }
    return is_1.isTimeString;
}
exports.default = DateTimeDef;
//# sourceMappingURL=datetime.js.map