import { ParserTreeValue } from '../parser/index';
import MemberDef from './memberdef';
import TypeDef from './typedef';
import KeyValueCollection from '../header';
/**
 * Represents the StringTypeDef which is reponsible for parsing,
 * validating, loading and serializing strings.
 *
 * It performs the following validation
 * - Value is string
 * - Value is optional
 * - Value is nullable
 * - Value length <= maxLength
 * - Value length >= minLength
 * - Value is in choices
 */
export default class StringDef implements TypeDef {
    private _type;
    constructor(type?: string);
    /**
     * Returns the type this instance is going to handle.
     * The return value could be any of the "string", "email", or "url"
     */
    getType(): string;
    /**
     * Parses the string in IO format into JavaScript strings.
     */
    parse(data: ParserTreeValue, memberDef: MemberDef, vars?: KeyValueCollection): string;
    /**
     * Loads the JavaScript string.
     */
    load(data: any, memberDef: MemberDef): string;
    /**
     * Serializes the string into IO format.
     */
    serialize(data: string, memberDef: MemberDef): string;
    validate(data: any, memberDef: MemberDef, vars?: KeyValueCollection): string;
}
//# sourceMappingURL=string.d.ts.map