"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TypedefRegistry = void 0;
var io_error_1 = require("../errors/io-error");
var io_error_codes_1 = require("../errors/io-error-codes");
var defsList = [];
var defs = {};
var TypedefRegistry = /** @class */ (function () {
    function TypedefRegistry() {
    }
    /**
     * Registers the TypeDef for the specified type.
     * @param type The type name
     * @param typeDef The associated TypeDef ojbect
     */
    TypedefRegistry.register = function (type, typeDef) {
        if (defs[type] === undefined) {
            defs[type] = typeDef;
            defsList.push(type);
        }
    };
    /**
     * Unregisters the specified type from the registry.
     * @param type The type name
     */
    TypedefRegistry.unregister = function (type) {
        if (defs[type] !== undefined) {
            delete defs[type];
            var index = defsList.indexOf(type);
            defsList.splice(index, 1);
        }
    };
    Object.defineProperty(TypedefRegistry, "types", {
        /**
         * Gets the array of registered type names.
         */
        get: function () {
            return __spreadArrays(defsList);
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Returns the associated TypeDef object when the specified type
     * is found otherwise returns undefined.
     * @param type The registered type
     */
    TypedefRegistry.get = function (type) {
        var typeDef = defs[type];
        if (typeDef === undefined) {
            throw new io_error_1.InternetObjectError(io_error_codes_1.default.invalidType, type);
        }
        return typeDef;
    };
    TypedefRegistry.isRegisteredType = function (typeName) {
        return typeName in defs;
    };
    return TypedefRegistry;
}());
exports.TypedefRegistry = TypedefRegistry;
//# sourceMappingURL=typedef-registry.js.map