"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.doCommonTypeCheck = void 0;
var io_error_codes_1 = require("../errors/io-error-codes");
var io_error_1 = require("../errors/io-error");
var is_1 = require("../utils/is");
/**
 * Performs the common validations required before serialization and deserialization
 * @param memberDef The memberDef object
 * @param value The value which needs to be validated
 * @param node The node object, required for tracing line and column when parsing raw internet object code!
 *
 * @internal
 */
function doCommonTypeCheck(memberDef, value, node) {
    var isUndefined = value === undefined;
    var isNull = is_1.isToken(node) ? node.value === null : value === null;
    // console.warn(">>>", JSON.stringify(memberDef, null, 2), value)
    // Check for undefined
    if (isUndefined) {
        if (memberDef.default)
            return _default(memberDef.default);
        if (memberDef.optional)
            return undefined;
        throw new (io_error_1.InternetObjectValidationError.bind.apply(io_error_1.InternetObjectValidationError, __spreadArrays([void 0], _valueRequired(memberDef, node))))();
    }
    // Check for null
    if (isNull) {
        if (memberDef.null)
            return null;
        throw new (io_error_1.InternetObjectError.bind.apply(io_error_1.InternetObjectError, __spreadArrays([void 0], _nullNotAllowed(memberDef, node))))();
    }
    // Validate choices
    if (memberDef.choices !== undefined && memberDef.choices.indexOf(value) === -1) {
        throw new (io_error_1.InternetObjectValidationError.bind.apply(io_error_1.InternetObjectValidationError, __spreadArrays([void 0], _invlalidChoice(memberDef, value, node))))();
    }
    // If everything is okay, return same data
    return value;
}
exports.doCommonTypeCheck = doCommonTypeCheck;
function _default(value) {
    if (is_1.isString(value)) {
        if (value === 'N')
            return null;
        if (value === 'T' || value === 'true')
            return true;
        if (value === 'F' || value === 'false')
            return false;
        return value;
    }
    return value;
}
function _valueRequired(memberDef, node) {
    return [io_error_codes_1.default.valueRequired, "Value is missing for '" + memberDef.path + "'.", node];
}
function _nullNotAllowed(memberDef, node) {
    return [io_error_codes_1.default.nullNotAllowed, memberDef.path + " does not support null."];
}
// Return an invalid choice error parameters
function _invlalidChoice(memberDef, value, node) {
    if (!memberDef.choices)
        throw Error('Choices not checked during NumberDef implementation.');
    return [
        io_error_codes_1.default.invalidChoice,
        "The value of \"" + memberDef.path + "\" must be one of the [" + memberDef.choices.join(',') + "]. Currently it is " + value + ".",
        node
    ];
}
//# sourceMappingURL=utils.js.map