"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.dateToTimeString = exports.dateToDateString = exports.dateToDatetimeString = exports.parseTime = exports.parseDate = exports.parseDateTime = exports.datetimePlainExp = exports.datetimeExp = void 0;
var is_1 = require("./is");
exports.datetimeExp = {
    datetime: /^((\d{4})(?:\-((?:1[0-2]|0[1-9]))(?:\-([0-2][0-9]|3[0-1]))?)?(T([0-1][0-9]|2[0-3])(?:\:([0-5][0-9])(?:\:([0-5][0-9])(?:\.(\d{3}))?)?)?)?((Z)|((?:\+|-)(?:(?:[0-1][0-9]|2[0-3])(?:\:[0-5][0-9])?)))?)$/,
    date: /^((\d{4})(?:\-((?:1[0-2]|0[1-9]))(?:\-([0-2][0-9]|3[0-1]))?)?)$/,
    time: /^(([0-1][0-9]|2[0-3])(?:\:([0-5][0-9])(?:\:([0-5][0-9])(?:\.(\d{3}))?)?)?)$/
};
exports.datetimePlainExp = {
    datetime: /^((\d{4})(?:((?:1[0-2]|0[1-9]))(?:([0-2][0-9]|3[0-1]))?)?(T([0-1][0-9]|2[0-3])(?:([0-5][0-9])(?:([0-5][0-9])(?:\.(\d{3}))?)?)?)?((Z)|((?:\+|-)(?:(?:[0-1][0-9]|2[0-3])(?:[0-5][0-9])?)))?)$/,
    date: /^((\d{4})(?:((?:1[0-2]|0[1-9]))(?:([0-2][0-9]|3[0-1]))?)?)$/,
    time: /^(([0-1][0-9]|2[0-3])(?:([0-5][0-9])(?:([0-5][0-9])(?:\.(\d{3}))?)?)?)$/
};
/**
 * Parses the value string and returns the datetime if the string represents
 * the ISO 8601 formatted datetime. Returns null when the invalid datetime
 * is found.
 */
exports.parseDateTime = function (value) {
    var exp = /[\-\:]/.test(value) ? exports.datetimeExp.datetime : exports.datetimePlainExp.datetime;
    var d = exp.exec(value);
    if (d === null) {
        return null;
    }
    // When time signature is not defined, makr the date as utc date
    // otherwise read the utc signature from the value.
    var utc = is_1.isUndefined(d[5]) ? 'Z' : d[10] || '';
    var dateStr = d[2] + "-" + (d[3] || '01') + "-" + (d[4] || '01') +
        ("T" + (d[6] || '00') + ":" + (d[7] || '00') + ":" + (d[8] || '00') + "." + (d[9] || '000') + utc);
    return new Date(dateStr);
};
/**
 * Parses the value string and returns the datetime if the string represents
 * the ISO 8601 formatted date. Returns null when the invalid date
 * is found.
 */
exports.parseDate = function (value) {
    var exp = /\-/.test(value) ? exports.datetimeExp.date : exports.datetimePlainExp.date;
    var d = exp.exec(value);
    if (d === null) {
        return null;
    }
    var dateStr = d[2] + "-" + (d[3] || '01') + "-" + (d[4] || '01');
    return new Date(dateStr);
};
/**
 * Parses the value string and returns the datetime if the string represents
 * the ISO 8601 formatted time. Returns null when the invalid time
 * is found.
 */
exports.parseTime = function (value) {
    var exp = /\:/.test(value) ? exports.datetimeExp.time : exports.datetimePlainExp.time;
    var d = exp.exec(value);
    if (d === null) {
        return null;
    }
    var dateStr = "1900-01-01T" + d[2] + ":" + (d[3] || '00') + ":" + (d[4] || '00') + "." + (d[5] || '000');
    return new Date(dateStr);
};
exports.dateToDatetimeString = function (date, noSep, zuluTime) {
    if (noSep === void 0) { noSep = false; }
    if (zuluTime === void 0) { zuluTime = false; }
    if (date === null)
        return null;
    if (zuluTime) {
        if (noSep)
            return date.toJSON().replace(/[\:\-]/g, '');
        return date.toJSON();
    }
    var Y = date.getFullYear();
    var M = _(date.getMonth() + 1);
    var D = _(date.getDate());
    var h = _(date.getHours());
    var m = _(date.getMinutes());
    var s = _(date.getSeconds());
    var ms = _(date.getMilliseconds(), 3);
    if (noSep)
        return "" + Y + M + D + "T" + h + m + s + "." + ms;
    return Y + "-" + M + "-" + D + "T" + h + ":" + m + ":" + s + "." + ms;
};
exports.dateToDateString = function (date, noSep) {
    if (noSep === void 0) { noSep = false; }
    if (date === null)
        return null;
    var Y = date.getFullYear();
    var M = _(date.getMonth() + 1);
    var D = _(date.getDate());
    if (noSep)
        return "" + Y + M + D;
    return Y + "-" + M + "-" + D;
};
exports.dateToTimeString = function (date, noSep) {
    if (noSep === void 0) { noSep = false; }
    if (date === null)
        return null;
    var h = _(date.getHours());
    var m = _(date.getMinutes());
    var s = _(date.getSeconds());
    var ms = _(date.getMilliseconds(), 3);
    if (noSep)
        return "" + h + m + s + "." + ms;
    return h + ":" + m + ":" + s + "." + ms;
};
var _ = function (n, pad) {
    if (pad === void 0) { pad = 2; }
    return n.toLocaleString('en-US', { minimumIntegerDigits: pad, useGrouping: false });
};
//# sourceMappingURL=datetime.js.map