"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isSchemaDef = exports.isKeyVal = exports.isDefined = exports.isNode = exports.isToken = exports.isParserTree = exports.isDataType = exports.isPlainObject = exports.isArray = exports.isTimeString = exports.isDateString = exports.isDateTimeString = exports.isDTTypeString = exports.isDate = exports.isBoolean = exports.isString = exports.isNumber = exports.isUndefined = void 0;
var typedef_registry_1 = require("../types/typedef-registry");
var datetime_1 = require("./datetime");
/**
 * Checks whether specified value is undefined or not.
 *
 * @param val The value to be tested for undefined check!
 * @returns `true` if the value is undefined otherwise `false`
 */
exports.isUndefined = function (val) {
    var _undefined = undefined;
    return val === _undefined;
};
/**
 * Checks whether specified value is number or not.
 *
 * @param val The value to be tested for number check!
 * @returns `true` if the value is a number otherwise `false`
 */
exports.isNumber = function (val) {
    return typeof val === 'number';
};
/**
 * Checks whether specified value is string or not.
 *
 * @param val The value to be tested for string check
 * @returns `true` if the value is a string otherwise `false`
 */
exports.isString = function (val) {
    return typeof val === 'string';
};
/**
 * Checks whether specified value is boolean or not.
 *
 * @param val The value to be tested for number check!
 * @returns `true` if the value is a boolean otherwise `false`
 */
exports.isBoolean = function (val) {
    return typeof val === 'boolean';
};
/**
 * Checks whether specified value is date or not.
 *
 * @param val The value to be tested for date check!
 * @returns `true` if the value is a date otherwise `false`
 */
exports.isDate = function (val) {
    return val instanceof Date;
};
/**
 * Checks whether specified value is date or not.
 *
 * @param val The value to be tested for date check!
 * @returns `true` if the value is a date otherwise `false`
 */
exports.isDTTypeString = function (val) {
    return exports.isDateTimeString(val) || exports.isDateString(val) || exports.isTimeString(val);
};
/**
 * Checks whether specified value is datetime or not.
 *
 * @param val The value to be tested for number check!
 * @returns `true` if the value is a datetime otherwise `false`
 */
exports.isDateTimeString = function (val) {
    var exp = /[\-\:]/.test(val) ? datetime_1.datetimeExp.datetime : datetime_1.datetimePlainExp.datetime;
    return exp.test(val);
};
/**
 * Checks whether specified value is date or not.
 *
 * @param val The value to be tested for date check!
 * @returns `true` if the value is a date otherwise `false`
 */
exports.isDateString = function (val) {
    var exp = /\-/.test(val) ? datetime_1.datetimeExp.date : datetime_1.datetimePlainExp.date;
    return exp.test(val);
};
/**
 * Checks whether specified value is time or not.
 *
 * @param val The value to be tested for time check!
 * @returns `true` if the value is a date otherwise `false`
 */
exports.isTimeString = function (val) {
    var exp = /\:/.test(val) ? datetime_1.datetimeExp.time : datetime_1.datetimePlainExp.time;
    return exp.test(val);
};
/**
 * Checks whether specified value is an array or not.
 *
 * @param val The value to be tested for an array check!
 * @returns `true` if the value is an array otherwise `false`
 */
exports.isArray = function (val) {
    return val instanceof Array;
};
exports.isPlainObject = function (val) {
    if (typeof val !== 'object')
        return false;
    // If has modified constructor
    var ctor = val.constructor;
    if (typeof ctor !== 'function')
        return false;
    // If has modified prototype
    var prot = ctor.prototype;
    if (typeof prot !== 'object')
        return false;
    // If constructor does not have an Object-specific method
    if (prot.hasOwnProperty('isPrototypeOf') === false) {
        return false;
    }
    // Most likely a plain Object
    return true;
};
/**
 * Checks whether specified value reprents datatype.
 *
 * @param val The value to be tested for an datatype check!
 * @returns `true` if the value is a datatype otherwise `false`
 */
exports.isDataType = function (val) {
    return typedef_registry_1.TypedefRegistry.isRegisteredType(val);
};
function isParserTree(obj) {
    try {
        return 'type' in obj && 'values' in obj;
    }
    catch (_a) {
        return false;
    }
}
exports.isParserTree = isParserTree;
/**
 * Checks whether the specified object is a Token or not!
 * @param obj The object to be validated for the Token check
 * @returns `true` if the value is a Token otherwise `false`
 */
function isToken(obj) {
    try {
        return obj && obj.token !== undefined && obj.value !== undefined;
    }
    catch (_a) {
        return false;
    }
}
exports.isToken = isToken;
/**
 * Checks whether the specified object is a Token or not!
 * @param obj The object to be validated for the Token check
 * @returns `true` if the value is a Token otherwise `false`
 */
function isNode(obj) {
    try {
        return obj && obj.index !== undefined && obj.row !== undefined && obj.col !== undefined;
    }
    catch (_a) {
        return false;
    }
}
exports.isNode = isNode;
function isDefined(val) {
    return val !== null && val !== undefined;
}
exports.isDefined = isDefined;
/**
 * Checks whether the specified object is a KeyVal or not!
 * @param obj The object to be validated for the KeyVal check
 * @returns `true` if the value is a KeyVal otherwise `false`
 */
function isKeyVal(obj) {
    try {
        return 'key' in obj && 'value' in obj;
    }
    catch (_a) {
        return false;
    }
}
exports.isKeyVal = isKeyVal;
/**
 *
 * @param obj
 */
exports.isSchemaDef = function (obj) {
    if (exports.isArray(obj)) {
        return typeof obj[0] === 'string' && exports.isDataType(obj[0]);
    }
};
// export function isToken(val: any): val is Token {
//   if (!val) return false
//   if (typeof val !== 'obj') return false
//   const keys = Object.keys(val)
// }
//# sourceMappingURL=is.js.map